"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path
import re
import os
import subprocess
import site

version_py = os.path.join(os.path.dirname(__file__), 'PyFd', '_version.py')

def virgo_version():
    """
    This function atempts to get the version from the git tag or from the parent directory
    name and writes the version to a _version.py file for use later. It assumes the format
    of the tag is  v#r#p#.
    """

    #Try to get the version from git or from the directory
    try:
       version = subprocess.check_output(["git", "describe","--tag"]).decode('utf8').rstrip()
    except:
       version = os.path.basename(os.path.dirname(os.path.realpath(__file__)))


    #convert v1r2p3 to 1.2.3 and v4r5 to 4.5.0
    match = re.match('v(\d+)r(\d+)(?:p(\d+))?', version)
    if match:
        version = '.'.join(match.groups('0'))  # assume p0 if missing

        #Write the version to a file so can be used later
        version_msg = "# Do not edit this file, pipeline versioning is governed by git tags"
        with open(version_py, 'w') as f:
                f.write(version_msg + os.linesep + ("__version__=\"%s\"" % version))
    elif(os.getenv("PKG_VERSION") is not None): # If in a conda environment
        version = os.getenv("PKG_VERSION")

        #Write the version to a file so can be used later
        version_msg = "# Do not edit this file, pipeline versioning is governed by git tags"
        with open(version_py, 'w') as f:
                f.write(version_msg + os.linesep + ("__version__=\"%s\"" % version))
    else:  # not a standard version directory or not a git respository
        if(os.path.exists(version_py)):
            with open(version_py, 'r') as f:
                version = f.read().strip().split('=')[-1].replace('"','')
        else:
            version="99.99.99"

    return version

#Find if the library exists in the expected location
def find_lib_loc():
    """Function to find the location of the libPyCm.so and install it so it can be imported"""

    #Check if we can find the library
    uname = os.getenv("UNAME","Linux-x86_64-CL7")
    if(os.path.exists(os.path.join(uname,"lib","libpyfd.so"))):
         lib_loc = os.path.join(uname,"lib","libpyfd.so")
    elif(os.path.exists(os.path.join("_build","src","libpyfd.so"))):
         lib_loc = os.path.join("_build","src","libpyfd.so") # built via conda
    elif(os.path.exists(os.path.join("..","..","libpyfd.so"))):
         lib_loc = "" # built via conda
    else:
         output = subprocess.check_output('find -name libpyfd.so',shell=True).splitlines()
         if(len(output)>0):
             lib_loc = output[0]
         else:
             raise IOError("The libpyfd.so cannot be found, has this code been compiled")

    #Get the install location check if being installed using pip or using "python setup.py install"
    using_pip = os.path.basename(os.path.dirname(__file__)).startswith('pip-')
    if(using_pip):
         install_loc = site.getusersitepackages() + "/PyFd"
    else:
          install_loc = "PyFd"
    if(lib_loc):
        return [(install_loc,[lib_loc])]
    else:
        return []

setup(
    name='PyFd',
    version=virgo_version(),
    description='Fd library Python binding via ctypegen',
    author='Franco carbognani',
    author_email='franco.carbognani@ego-gw.it',
    url='http://www.virgo-gw.eu/',
    tests_require = [
        "pytest"
    ],
    packages=["PyFd"],
    data_files = find_lib_loc(),
)
